<?php

namespace App\Service;

use App\Model\BasicType;
use App\Model\Depart;
use App\Model\Employee;
use App\Model\PurchaseOrder;
use App\Model\PurchaseOrderInfo;
use App\Model\SalesOrder;
use App\Model\Storehouse;
use App\Model\Supplier;
use Illuminate\Support\Facades\DB;

/**
 * 采购单
 */
class PurchaseOrderService extends Service
{
    /**
     * 采购单编辑
     * @param $data
     * @param $user
     * @return array
     */
    public function edit($data,$user){
        list($status,$msg) = $this->orderRule($data,$user, false);
        if(!$status) return [$status,$msg];

        try{
            DB::beginTransaction();
            $material_model = PurchaseOrder::where('order_number',$data['order_number'])->first();
            $material_model->order_number = $data['order_number'];
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->top_depart_id = $data['top_depart_id'] ?? 0;
            $material_model->total = $data['total'] ?? 0;
            $material_model->other_fee = $data['other_fee'] ?? 0;
            $material_model->discount_fee = $data['discount_fee'] ?? 0;
            $material_model->purchase_total = $data['purchase_total'] ?? 0;
            $material_model->supplier = $data['supplier'] ?? 0;
            $material_model->purchase_id = $data['purchase_id'] ?? 0;
            $material_model->order_type = $data['order_type'];
            $material_model->sales_order_id = $data['sales_order_id'] ?? 0;
            $material_model->mark = $data['mark'] ?? '';
            $material_model->storehouse_id = $data['storehouse_id'];
            $material_model->save();

            PurchaseOrderInfo::where('order_number',$data['order_number'])
                ->where('del_time',0)
                ->update(['del_time' => time()]);
            if(!empty($data['product'])){
                $sub = [];
                foreach ($data['product'] as $value){
                    $sub[] = [
                        'purchase_order_id' => $material_model->id,
                        'order_number' => $data['order_number'],
                        'storehouse_id' => $data['storehouse_id'] ?? 0,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                        'mark' => $value['mark'] ?? '',
                    ];
                }
                PurchaseOrderInfo::insert($sub);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 采购单新增
     * @param $data
     * @param $user
     * @return array
     */
    public function add($data,$user){
        list($status,$msg) = $this->orderRule($data, $user);
        if(!$status) return [$status,$msg];
        try{
            DB::beginTransaction();
            $material_model = new PurchaseOrder();
            $material_model->order_number = $data['order_number'];
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->top_depart_id = $data['top_depart_id'] ?? 0;
            $material_model->total = $data['total'] ?? 0;
            $material_model->other_fee = $data['other_fee'] ?? 0;
            $material_model->discount_fee = $data['discount_fee'] ?? 0;
            $material_model->purchase_total = $data['purchase_total'] ?? 0;
            $material_model->supplier = $data['supplier'] ?? 0;
            $material_model->purchase_id = $data['purchase_id'] ?? 0;
            $material_model->order_type = $data['order_type'];
            $material_model->sales_order_id = $data['sales_order_id'] ?? 0;
            $material_model->mark = $data['mark'] ?? '';
            $material_model->crt_id = $user['id'];
            $material_model->storehouse_id = $data['storehouse_id'];
            $material_model->save();

            if(!empty($data['product'])){
                $sub = [];
                foreach ($data['product'] as $value){
                    $sub[] = [
                        'purchase_order_id' => $material_model->id,
                        'order_number' => $data['order_number'],
                        'storehouse_id' => $data['storehouse_id'] ?? 0,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                        'mark' => $value['mark'] ?? '',
                    ];
                }
                PurchaseOrderInfo::insert($sub);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 采购单详情
     * @param $data
     * @return array
     */
    public function detail($data){
        if($this->isEmpty($data,'order_number')) return [false,'请选择数据'];

        $order = PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->select('id','order_number','depart_id','mark','state','crt_id','order_type','sales_order_id','supplier','purchase_id','total','other_fee','discount_fee','purchase_total','crt_time','storehouse_id')
            ->first();
        if(empty($order)) return [false, '采购订单不存在或已被删除'];
        $order = $order->toArray();
        $order['storehouse_title'] = Storehouse::where('id',$order['storehouse_id'])->value('title');
        $order['supplier_title'] = Supplier::where('id',$order['supplier'])->value('title');
        $order['order_type_title'] = PurchaseOrder::$order_type[$order['order_type']] ?? '';
        $order['order_type_title'] = PurchaseOrder::$order_type[$order['order_type']] ?? '';
        $order['state_title'] = PurchaseOrder::$name[$order['state']] ?? '';
        $sales = SalesOrder::where('id',$order['sales_order_id'])->value('order_number');
        $order['sales_order_number'] = $sales;
        $emp_map = Employee::whereIn('id',array_unique(array_merge_recursive([$order['purchase_id']],[$order['crt_id']])))
            ->pluck('emp_name','id')
            ->toArray();
        $order['purchase_title'] = $emp_map[$order['purchase_id']] ?? '';

        $p_info = PurchaseOrderInfo::where('del_time',0)
            ->where('order_number',$data['order_number'])
            ->get()->toArray();
        $basic_price = BasicType::whereIn('id',array_unique(array_column($p_info,'basic_type_id')))->pluck('title','id')->toArray();
        $map = (new ProductService())->getProductDetail(array_column($p_info,'product_id'));
        foreach ($p_info as $value){
            $tmp = $map[$value['product_id']] ?? [];
            $value['title'] = $tmp['title'] ?? "";
            $value['code'] = $tmp['code'] ?? "";
            $value['size'] = $tmp['size'] ?? "";
            $value['unit'] = $tmp['unit'] ?? "";
            $value['bar_code'] = $tmp['bar_code'] ?? "";
            $value['basic_type_title'] = $basic_price[$value['basic_type_id']] ?? "";
            $order['product'][] = $value;
        }
        $order['crt_name'] = $emp_map[$order['crt_id']] ?? '';
        $order['crt_time'] = $order['crt_time'] ? date("Y-m-d H:i:s",$order['crt_time']): '';

        return [true, $order];
    }

    /**
     * 采购单删除
     * @param $data
     * @return array
     */
    public function del($data){
        if($this->isEmpty($data,'order_number')) return [false,'请选择数据！'];
        $order = PurchaseOrder::where('del_time',0)->where('order_number',$data['order_number'])->first();
        if(empty($order)) return [false,'采购单不存在或已被删除'];
        $order = $order->toArray();
        if($order['state'] > PurchaseOrder::STATE_ZERO) return [false,'请确认采购订单状态,删除失败！'];

        try{
            DB::beginTransaction();
            PurchaseOrder::where('order_number',$data['order_number'])->update([
                'del_time'=>time()
            ]);
            PurchaseOrderInfo::where('order_number',$data['order_number'])->update([
                'del_time'=>time()
            ]);

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 获取采购单列表
     * @param $data
     * @param $user
     * @return array
     */
    public function getList($data,$user){
        $model = new PurchaseOrder(['userData' => $user, 'search' => $data]);
        $model = $model->where('del_time',0)->select('order_number','id','depart_id','top_depart_id','mark','crt_time','state','crt_id','order_type','sales_order_id','supplier','purchase_id','purchase_total','discount_fee','other_fee','total','storehouse_id')
            ->orderby('id', 'desc');
        if(! empty($data['order_number'])) $model->where('order_number', 'LIKE', '%'.$data['order_number'].'%');
        if(! empty($data['depart_id'])) $model->where('depart_id', $data['depart_id']);
        if(! empty($data['company_id'])) $model->where('company_id', $data['company_id']);
        if(isset($data['state'])) $model->where('state',$data['state']);
        if(! empty($data['crt_time'][0]) && ! empty($data['crt_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['crt_time']);
            $model->whereBetween('crt_time',[$return[0],$return[1]]);
        }
        if(! empty($data['time_type'])) {
            if($data['time_type'] == 1) {
                $start = strtotime('today');
                $end = strtotime('tomorrow') - 1;
            }elseif ($data['time_type'] == 2){
                $start = strtotime('this week',strtotime('today'));
                $end = strtotime('this week +6 days 23:59:59', strtotime('today'));
            }
            if(! empty($start) && ! empty($end)) {
                $model->where('crt_time','>=',$start);
                $model->where('crt_time','<=',$end);
            }
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillListData($list);

        return [true, $list];
    }

    /**
     * 拼接数据
     * @param $data
     * @return array
     */
    public function fillListData($data){
        if(empty($data['data'])) return $data;

        $depart_id = array_unique(array_column($data['data'],'depart_id'));
        $depart_map = Depart::whereIn('id',$depart_id)
            ->pluck('title','id')
            ->toArray();

        $emp_id = array_unique(array_merge_recursive(array_column($data['data'],'crt_id'),array_column($data['data'],'purchase_id')));
        $emp_map = Employee::whereIn('id',$emp_id)
                    ->pluck('emp_name','id')
                    ->toArray();

        $sales = SalesOrder::whereIn('id',array_unique(array_column($data['data'],'sales_order_id')))->pluck('order_number','id')->toArray();
        $storehouse = Storehouse::whereIn('id',array_unique(array_column($data['data'],'storehouse_id')))
            ->pluck('title','id')
            ->toArray();
        $supplier = Supplier::whereIn('id',array_unique(array_column($data['data'],'supplier')))
            ->pluck('title','id')
            ->toArray();
        foreach ($data['data'] as $key => $value){
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date("Y-m-d H:i:s",$value['crt_time']) : '';
            $data['data'][$key]['depart_name'] = $depart_map[$value['depart_id']] ?? '';
            $data['data'][$key]['crt_name'] = $emp_map[$value['crt_id']] ?? '';
            $data['data'][$key]['purchase_name'] = $emp_map[$value['purchase_id']] ?? '';
            $data['data'][$key]['order_type_title'] = PurchaseOrder::$order_type[$value['order_type']] ?? '';
            $data['data'][$key]['state_title'] = PurchaseOrder::$name[$value['state']] ?? '';
            $data['data'][$key]['sales_order_number'] = $sales[$value['sales_order_id']] ?? '';
            $data['data'][$key]['storehouse_title'] = $storehouse[$value['storehouse_id']] ?? '';
            $data['data'][$key]['supplier_title'] = $supplier[$value['supplier']] ?? '';
        }

        return $data;
    }

    /**
     * 采购单参数规则
     * @param $data
     * @param $is_check
     * @return array
     */
    public function orderRule(&$data, $user, $is_check = true){
        if(empty($data['order_type'])) return [false,'采购订单类型不能为空'];
        if($data['order_type'] == PurchaseOrder::Order_type_two && empty($data['sales_order_id'])) return [false, PurchaseOrder::$order_type[PurchaseOrder::Order_type_two] . '必须选择合同'];
        if($this->isEmpty($data,'product')) return [false,'产品不能为空'];
        if(empty($data['storehouse_id'])) return [false,'请选择仓库'];
        if(! empty($data['other_fee'])){
            $res = $this->checkNumber($data['other_fee']);
            if(! $res) return [false, '其它费用请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['discount_fee'])){
            $res = $this->checkNumber($data['discount_fee']);
            if(! $res) return [false, '优惠金额请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['total'])){
            $res = $this->checkNumber($data['total']);
            if(! $res) return [false, '产品总价请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['purchase_total'])){
            $res = $this->checkNumber($data['purchase_total']);
            if(! $res) return [false, '采购总价请输入不超过两位小数并且大于0的数值'];
        }
        foreach ($data['product'] as $value){
            if(empty($value['number'])) return [false,'产品数量不能为空'];
            $res = $this->checkNumber($value['number']);
            if(! $res) return [false,'请输入正确的产品数量'];
        }

        //所属部门 以及 顶级部门
        if(empty($data['depart_id'])) $data['depart_id'] = $this->getDepart($user);
        $data['top_depart_id'] = $user['depart_map'][$data['depart_id']] ?? 0;

        if($is_check){
            $order_number = (new OrderNoService())->createOrderNumber(PurchaseOrder::prefix);
            if(empty($order_number)) return [false,'采购单号生成失败！'];
            $data['order_number'] = $order_number;
        }else{
            if($this->isEmpty($data,'order_number')) return [false,'采购单号不能为空！'];
        }

        return [true, ''];
    }

    /**
     * 采购单确认
     * @param $data
     * @param $user
     * @return array
     */
    public function purchaseOrderConfirm($data,$user){
        if(empty($data['order_number'])) return [false, '请选择数据'];

        $order = PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->first();
        if(empty($order)) return [false, '采购订单不存在或已被删除'];
        if($order->state != PurchaseOrder::STATE_ZERO) return [false, '请确认采购订单状态，操作失败'];

        PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->update(['state' => PurchaseOrder::STATE_ONE]);

        return [true,''];
    }
}
